"""

Antix: Antyx is an automated EDA engine designed to generate fast, structured and professional exploratory reports from any tabular dataset.

- Upload data (CSV/Excel/...).
- Automathic clean.
- Exploratory Data Analysis.
- and more
"""

__version__ = "0.1"
__name__ = "antyx"
__author__ = "Daniel Rodrigálvarez Morente"
__email__ = "drm.datos@gmail.com"
